@@ -1,20 +1,23 @@
 ## The BUGSdeclClass contains the pulled-apart content of a BUGS declaration line
 
-## nimbleOrRfunctionNames is used to determine what can be evaluated in R if every argument is known OR in C++ (nimble) if arguments are other nodes
+## nimbleOrRfunctionNames is used to determine what (in BUGS code) can be evaluated in R if every argument is known OR in C++ (nimble) if arguments are other nodes
 nimbleOrRfunctionNames <- c('[','+','-','/','*','(','exp','log','pow','^','%%','%*%','t',
                             'equals','inprod','nimEquals',
                             'sqrt', 'logit', 'expit', 'ilogit', 'probit', 'iprobit', 'phi', 'cloglog', 'icloglog', 'step', 'nimStep',
                             'sin','cos','tan','asin','acos','atan','cosh','sinh','tanh', 'asinh', 'acosh', 'atanh',
                             'cube', 'abs', 'lgamma', 'loggam', 'log1p', 'lfactorial', ##'factorial', 'gamma',
-                            'ceiling', 'floor', 'round', 'nimRound', 'trunc',
+                            'ceiling', 'floor', 'round', 'nimRound', 'optim', 'nimOptim', 'trunc',
                             'mean','sum','sd','var','max','min','prod',
                             'asRow', 'asCol',
-                            'chol', 'inverse', 'forwardsolve', 'backsolve', 'solve', 'nimEigen', 'nimSvd',  ## removed these from BUGS functions, pending problems with Eigen
+                            'chol', 'inverse', 'forwardsolve', 'backsolve', 'solve', 'nimEigen', 'nimSvd', 'nimOptim',  ## removed these from BUGS functions, pending problems with Eigen
                             '>', '<', '>=', '<=', '==', '!=', '&', '|', '$',
                             distributionFuns,
                             # these are allowed in DSL as special cases even though exp_nimble and t_nonstandard are the canonical NIMBLE distribution functions
                             paste0(c('d','r','q','p'), 't'),
-                            paste0(c('d','r','q','p'), 'exp'))
+                            paste0(c('d','r','q','p'), 'exp'),
+                            'nimC', 'nimRep', 'nimSeq', 'diag')
+
+functionsThatShouldNeverBeReplacedInBUGScode <- c(':','nimC','nimRep','nimSeq', 'diag')
 
 #' BUGSdeclClass contains the information extracted from one BUGS declaration
 BUGSdeclClass <- setRefClass('BUGSdeclClass',
@@ -478,8 +481,10 @@ genReplacementsAndCodeRecurse <- function(code, constAndIndexNames, nimbleFuncti
             contentsReplaceable  <- list()
             allContentsReplaceable <- TRUE
         }
-        if(code[[1]] == ':')   return(replaceWhatWeCan(code, contentsCodeReplaced, contentsReplacements, contentsReplaceable, startingAt=2)) ## for newNodeFxns, use default replaceable = FALSE for any ':' expression.  old: , replaceable=allContentsReplaceable))
-        if(assignment)         return(replaceWhatWeCan(code, contentsCodeReplaced, contentsReplacements, contentsReplaceable, startingAt=2))
+        if(deparse(code[[1]]) %in% functionsThatShouldNeverBeReplacedInBUGScode)
+            return(replaceWhatWeCan(code, contentsCodeReplaced, contentsReplacements, contentsReplaceable, startingAt=2))
+        if(assignment)
+            return(replaceWhatWeCan(code, contentsCodeReplaced, contentsReplacements, contentsReplaceable, startingAt=2))
         isRfunction <- !any(code[[1]] == nimbleFunctionNames)
         isRonly <- isRfunction & !checkNimbleOrRfunctionNames(deparse(code[[1]]))
         if(deparse(code[[1]]) == '$') isRonly <- FALSE
