@@ -18,7 +18,9 @@ RCvirtualFunProcessing <- setRefClass('RCvirtualFunProcessing',
                                           RCfun = 'ANY', ##nfMethodRC
                                           nameSubList = 'ANY',
                                           compileInfo = 'ANY', ## RCfunctionCompileClass``
-                                          const = 'ANY'
+                                          const = 'ANY',
+                                          neededRCfuns = 'ANY',
+                                          initialTypeInferenceDone = 'ANY'
                                           ),
                                       methods = list(
                                           initialize = function(f = NULL, funName, const = FALSE) {
@@ -40,11 +42,13 @@ RCvirtualFunProcessing <- setRefClass('RCvirtualFunProcessing',
                                                   }
                                                   compileInfo <<- RCfunctionCompileClass$new(origRcode = RCfun$code, newRcode = RCfun$code)
                                               }
+                                              neededRCfuns <<- list()
+                                              initialTypeInferenceDone <<- FALSE
                                           },
                                           showCpp = function() {
                                               writeCode(nimGenerateCpp(compileInfo$nimExpr, compileInfo$newLocalSymTab))
                                           },
-                                          setupSymbolTables = function(parentST = NULL, neededTypes = NULL) {
+                                          setupSymbolTables = function(parentST = NULL, neededTypes = NULL, nimbleProject = NULL) {
                                               argInfoWithMangledNames <- RCfun$argInfo
                                               numArgs <- length(argInfoWithMangledNames)
                                               if(numArgs > 0) {
@@ -59,13 +63,25 @@ RCvirtualFunProcessing <- setRefClass('RCvirtualFunProcessing',
                                               if(numArgs>0) names(argInfoWithMangledNames) <- paste0("ARG", 1:numArgs, "_", Rname2CppName(names(argInfoWithMangledNames)),"_")
                                               nameSubList <<- lapply(names(argInfoWithMangledNames), as.name)
                                               names(nameSubList) <<- names(RCfun$argInfo)
+
+                                              ## This will only handle basic types.  nimbleLists will be added below
                                               compileInfo$origLocalSymTab <<- argTypeList2symbolTable(argInfoWithMangledNames, neededTypes, names(RCfun$argInfo)) ## will be used for function args.  must be a better way.
                                               compileInfo$newLocalSymTab <<- argTypeList2symbolTable(argInfoWithMangledNames, neededTypes, names(RCfun$argInfo))
+
+                                              ## This modifies the symTab and returns types needed for input or return
+                                              ## Currently the only non-basic type resolves in this step would be nimbleLists
+                                              if(is.null(nimbleProject)) nimbleProject <- get('nimbleProject', envir = RCfun)
+                                              neededRCfuns <<- resolveUnknownTypes(compileInfo$origLocalSymTab, neededTypes, nimbleProject)
+                                              resolveUnknownTypes(compileInfo$newLocalSymTab, c(neededRCfuns, neededTypes), nimbleProject)
+                                              
                                               if(!is.null(parentST)) {
                                                   compileInfo$origLocalSymTab$setParentST(parentST)
                                                   compileInfo$newLocalSymTab$setParentST(parentST)
                                               }
                                               compileInfo$returnSymbol <<- argType2symbol(RCfun$returnType, neededTypes, "return", "returnType")
+                                              updatedReturn <- resolveOneUnknownType(compileInfo$returnSymbol, c(neededRCfuns, neededTypes), nimbleProject)
+                                              compileInfo$returnSymbol <<- updatedReturn[[1]]
+                                              neededRCfuns <<- c(neededRCfuns, updatedReturn[[2]])
                                           },
                                           process = function(...) {
                                               if(inherits(compileInfo$origLocalSymTab, 'uninitializedField')) {
@@ -104,6 +120,7 @@ nf_substituteExceptFunctionsAndDollarSigns <- function(code, subList) {
     if(is.character(code)) return(code)
     if(is.numeric(code)) return(code)
     if(is.logical(code)) return(code)
+    if(is.null(code)) return(code)
     if(is.name(code)) {
         maybeAns <- subList[[as.character(code)]]
         return( if(is.null(maybeAns)) code else maybeAns )
@@ -128,11 +145,9 @@ nf_substituteExceptFunctionsAndDollarSigns <- function(code, subList) {
 
 RCfunProcessing <- setRefClass('RCfunProcessing',
                                contains = 'RCvirtualFunProcessing',
-                               fields = list(
-                                   neededRCfuns = 'list' ## nfMethodRC objects
-                                   ),
-                               methods = list(
-                                   process = function(debug = FALSE, debugCpp = FALSE, debugCppLabel = character(), doKeywords = TRUE) {
+                               fields = list(),
+                                   methods = list(
+                                   process = function(debug = FALSE, debugCpp = FALSE, debugCppLabel = character(), doKeywords = TRUE, nimbleProject = NULL, initialTypeInferenceOnly = FALSE) {
                                        if(!is.null(nimbleOptions()$debugRCfunProcessing)) {
                                            if(nimbleOptions()$debugRCfunProcessing) {
                                                debug <- TRUE
@@ -140,22 +155,33 @@ RCfunProcessing <- setRefClass('RCfunProcessing',
                                            }
                                        }
 
-                                       if(!is.null(nimbleOptions()$debugCppLineByLine)) {
-                                           if(nimbleOptions()$debugCppLineByLine) {
-                                               debugCpp <- TRUE
-                                               if(length(debugCppLabel) == 0) debugCppLabel <- name
+                                       if(debug) {
+                                           writeLines('**** READY to start RCfunProcessing::process *****')
+                                           browser()
+                                       }
+
+                                       if(is.null(nimbleProject)) nimbleProject <- get('nimbleProject', envir = RCfun)
+                                       
+                                       if(!initialTypeInferenceDone) {
+                                           
+                                           if(!is.null(nimbleOptions()$debugCppLineByLine)) {
+                                               if(nimbleOptions()$debugCppLineByLine) {
+                                                   debugCpp <- TRUE
+                                                   if(length(debugCppLabel) == 0) debugCppLabel <- name
+                                               }
                                            }
+                                           
+                                           if(doKeywords) {
+                                               matchKeywords()
+                                               processKeywords()
+                                           }
+                                           
+                                           if(inherits(compileInfo$origLocalSymTab, 'uninitializedField')) {
+                                               setupSymbolTables()
+                                           }
+                                           initialTypeInferenceDone <<- TRUE
                                        }
-
-                                       if(doKeywords) {
-                                           matchKeywords()
-                                           processKeywords()
-                                       }
-                                                                              
-                                       if(inherits(compileInfo$origLocalSymTab, 'uninitializedField')) {
-                                           setupSymbolTables()
-                                       }
-
+                                       if(initialTypeInferenceOnly) return(NULL);
                                       
                                        if(debug) {
                                            writeLines('**** READY FOR makeExprClassObjects *****')
@@ -206,14 +232,31 @@ RCfunProcessing <- setRefClass('RCfunProcessing',
                                        compileInfo$typeEnv[['neededRCfuns']] <<- list()
                                        compileInfo$typeEnv[['.AllowUnknowns']] <<- TRUE ## will be FALSE for RHS recursion in setSizes
                                        compileInfo$typeEnv[['.ensureNimbleBlocks']] <<- FALSE ## will be TRUE for LHS recursion after RHS sees rmnorm and other vector dist "r" calls.
+                                       compileInfo$typeEnv[['.nimbleProject']] <<- nimbleProject
                                        passedArgNames <- as.list(compileInfo$origLocalSymTab$getSymbolNames()) 
                                        names(passedArgNames) <- compileInfo$origLocalSymTab$getSymbolNames() 
-                                       compileInfo$typeEnv[['passedArgumentNames']] <<- passedArgNames ## only the names are used.  
-                                       tryResult <- try(exprClasses_setSizes(compileInfo$nimExpr, compileInfo$newLocalSymTab, compileInfo$typeEnv))
-                                       if(inherits(tryResult, 'try-error')) {
-                                           stop(paste('There is some problem at the setSizes processing step for this code:\n', paste(deparse(compileInfo$origRcode), collapse = '\n'), collapse = '\n'), call. = FALSE)
-                                       }
-                                       neededRCfuns <<- compileInfo$typeEnv[['neededRCfuns']]
+
+                                       compileInfo$typeEnv[['passedArgumentNames']] <<- passedArgNames ## only the names are used.
+                                       ## This attempts to prevent the traceback from being hidden by multiple layers of error trapping.
+                                       ## A better solution might be to avoid layered trapping so that options(error = recover) could function.
+                                       tryCatch(withCallingHandlers(exprClasses_setSizes(compileInfo$nimExpr, compileInfo$newLocalSymTab, compileInfo$typeEnv),
+                                                                    error = function(e) {
+                                                                        stack <- sapply(sys.calls(), deparse)
+                                                                        .GlobalEnv$.nimble.traceback <- capture.output(traceback(stack))
+                                                                    }),
+                                                error = function(e) {
+                                                    eMessage <- if(!is.null(e$message)) paste0(as.character(e$message),"\n") else ""
+                                                    message <- paste(eMessage, 'There was some problem in the the setSizes processing step for this code:',
+                                                                     paste(deparse(compileInfo$origRcode), collapse = '\n'))
+                                                    if(getNimbleOption('verboseErrors')) {
+                                                        message <- paste(message,
+                                                                         'Internal Error:', e,
+                                                                         'Traceback:', paste0(.GlobalEnv$.nimble.traceback, collapse = '\n'),
+                                                                         sep = '\n')
+                                                    }
+                                                    stop(message, call. = FALSE)
+                                                })
+                                       neededRCfuns <<- c(neededRCfuns, compileInfo$typeEnv[['neededRCfuns']])
                                        
                                        if(debug) {
                                            print('compileInfo$nimExpr$show(showType = TRUE) -- broken')
