@@ -1,6 +1,18 @@
 # for use in DSL code check:
-otherDSLcalls <- c("{", "[[", "$", "resize", "declare", "returnType", "seq_along", "double", "rankSample", "new",
-                   "nimEigen", "nimSvd")
+otherDSLcalls <- c("{",
+                   "[[",
+                   "$",
+                   "resize",
+                   "declare",
+                   "returnType",
+                   "seq_along",
+                   "double",
+                   "rankSample",
+                   "new",
+                   "nimEigen",
+                   "nimSvd",
+                   "nimOptim",
+                   "void")
 
 nimKeyWords <- list(copy = 'nimCopy',
                     print = 'nimPrint',
@@ -19,7 +31,8 @@ nimKeyWords <- list(copy = 'nimCopy',
                     rep = 'nimRep',
                     seq = 'nimSeq',
                     eigen = 'nimEigen',
-                    svd = 'nimSvd')
+                    svd = 'nimSvd',
+                    optim = 'nimOptim')
 
 nfMethodRCinterface <- setRefClass(Class = 'nfMethodRCinterface',
                                    fields = list(
@@ -136,9 +149,9 @@ nf_checkDSLcode <- function(code, methodNames, setupVarNames) {
                                      sapply(nonDSLinR, function(x) is.rcf(x, inputIsName = TRUE)) |
                                      sapply(nonDSLinR, function(x) is.nlGenerator(x, inputIsName = TRUE)))]
         if(length(nonDSLinR))
-           warning(paste0("Detected possible use of R functions in nimbleFunction run code. For this nimbleFunction to compile, these objects must be defined as nimbleFunctions, nimbleFunctionLists, or nimbleLists: ", paste(nonDSLinR, collapse = ', '), "."))
+           warning(paste0("Detected possible use of R functions in nimbleFunction run code. For this nimbleFunction to compile, these objects must be defined as nimbleFunctions, nimbleFunctionLists, or nimbleLists: ", paste(nonDSLinR, collapse = ', '), "."), call. = FALSE)
         if(length(nonDSLnonR))
-            warning(paste0("For this nimbleFunction to compile, these objects must be defined as nimbleFunctions, nimbleFunctionLists, or nimbleLists before compilation: ", paste(nonDSLnonR, collapse = ', '), "."))
+            warning(paste0("For this nimbleFunction to compile, these objects must be defined as nimbleFunctions, nimbleFunctionLists, or nimbleLists before compilation: ", paste(nonDSLnonR, collapse = ', '), "."), call. = FALSE)
     }
     return(0)
 }
