@@ -208,7 +208,9 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                                         'Utils.cpp',
                                                         'NamedObjects.cpp',
                                                         'ModelClassUtils.cpp',
-                                                        'accessorClasses.cpp'
+                                                        'accessorClasses.cpp',
+                                                        'predefinedNimbleLists.cpp',
+                                                        'nimOptim.cpp'
                                                         )
                                        if(getNimbleOption('includeCPPdists')) cppPermList <- c(cppPermList, 'dists.cpp', 'nimDists.cpp')
 
