@@ -29,6 +29,7 @@ CmodelValues <- setRefClass(
     fields = list(
         dll = 'ANY',
         extptr = 'ANY',
+        namedObjectsPtr = 'ANY',
         extptrCall = 'ANY',
         varNames = 'ANY',
         componentExtptrs = 'ANY',
@@ -62,20 +63,24 @@ CmodelValues <- setRefClass(
         finalizeInternal = function() {
             finalize()
             extptr <<- NULL
+            namedObjectsPtr <<- NULL
         },
         finalize = function() {
-            nimbleInternalFunctions$nimbleFinalize(extptr)
+            nimbleInternalFunctions$nimbleFinalize(namedObjectsPtr) ##
         },
         initialize = function(buildCall, existingPtr, initialized = FALSE, dll) {
             if(missing(existingPtr) ) {
                 if(is.character(buildCall)) {
                     warning("a call to getNativeSymbolInfo with only a name and no DLL")
                 }
+               
                 # avoid R CMD check problem with registration
                 ## notice that buildCall is the result of getNativeSymbolInfo using the dll from nimbleProject$instantiateCmodelValues
                 ## only other calling point is from cppInterfaces_models.R, and in that case existingPtr is provided
-                extptr <<- eval(parse(text = ".Call(buildCall)"))
-                eval(call('.Call',nimbleUserNamespace$sessionSpecificDll$register_namedObjects_Finalizer, extptr, dll[['handle']], 'modelValues'))
+                extptrlist <- eval(parse(text = ".Call(buildCall)"))
+                extptr <<- extptrlist[[1]]
+                namedObjectsPtr <<- extptrlist[[3]] ## order should come from the cppDef, but cheating here to get it right
+                eval(call('.Call',nimbleUserNamespace$sessionSpecificDll$register_namedObjects_Finalizer, namedObjectsPtr, dll[['handle']], 'modelValues'))
 #                extptr <<- .Call(buildCall)
             }
             else{
@@ -92,20 +97,6 @@ CmodelValues <- setRefClass(
             blankAns <<- componentExtptrs
             for(comp in varNames) 
                 componentExtptrs[[comp]] <<- eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getModelObjectPtr, extptr, comp))
-                
-            ## .nodePtrs_byGID <<- new('numberedObjects')
-            ## GID_map <<- makeMV_GID_Map(.self)
-            ## if(length(sizes) > 0){
-            ## 	varLengths <- sapply(sizes, prod)
-            ## 	totLength <- sum(varLengths)
-            ##     	.nodePtrs_byGID$resize(totLength)
-            ## 	index = 1
-            ## 	for(i in seq_along(varNames)){
-            ##     	  	vName <- varNames[i]
-            ##     	  	.Call('populateNumberedObject_withSingleModelValuesAccessors', extptr, vName, as.integer(expandNodeNames(vName, returnType = 'ids')), as.integer(1) , .nodePtrs_byGID$.ptr)
-            ## 		index = index + varLengths[i]
-            ## 	}
-            ## }
         },
         resize = function(rows){	
         	for(ptr in componentExtptrs)
@@ -124,50 +115,50 @@ setMethod('[', 'CmodelValues',
               if(missing(j) ) 
                   j = 1:cGetNRow(x)
               if(length(i) == 1){
-              	ptr = x$componentExtptrs[[i]]
-              	if(is.null(ptr) ) 
-              		stop(paste('variable', i, ' not found in modelValues') ) 
-              	if(length(j) == 1){
-              		output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElement, ptr, as.integer(j)) )
-              		return(output) 
-              		}
-              	output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElementAsList, ptr, as.integer(j)) )
-              	return(output)
-              	}
-             output <- list() 	
-             for(cmp in i)
-             	output[[cmp]] <- x[cmp, j]
-          return(output)
+                  ptr = x$componentExtptrs[[i]]
+                  if(is.null(ptr) ) 
+                      stop(paste('variable', i, ' not found in modelValues') ) 
+                  if(length(j) == 1){
+                      output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElement, ptr, as.integer(j)) )
+                      return(output) 
+                  }
+                  output = eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getMVElementAsList, ptr, as.integer(j)) )
+                  return(output)
+              }
+              output <- list() 	
+              for(cmp in i)
+                  output[[cmp]] <- x[cmp, j]
+              return(output)
           }
           )
 
 setMethod('[<-', 'CmodelValues',
-			function(x, i, j, value){
-				if(missing(i) ) 
-					i = x$varNames
-				if(missing(j) ) 
-					j = 1:getsize(x)
-				if(length(i) == 1){
-	              	ptr = x$componentExtptrs[[i]]
-	              	if(is.null(ptr) ) 
-	              		stop(paste('variable', i, ' not found in modelValues') ) 
-					if(length(j) == 1){
-						storage.mode(value) <- 'numeric'
-						eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElement, ptr, as.integer(j), value ))
-						return(x)
-					}
-				for(jj in j)
-					storage.mode(value[[jj]]) <- 'numeric'
-				eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElementFromList, ptr, value, as.integer(j)) )
-				return(x)
-				}
-			cmpNames = names(value)
-			if( !all(cmpNames %in% x$varNames) ) 
-				stop('Warning: names of modelValue elements do not match')
-			for(n in cmpNames)
-				x[n, j] <- value[[n]]
-			return(x)
-			})
+          function(x, i, j, value){
+              if(missing(i) ) 
+                  i = x$varNames
+              if(missing(j) ) 
+                  j = 1:getsize(x)
+              if(length(i) == 1){
+                  ptr = x$componentExtptrs[[i]]
+                  if(is.null(ptr) ) 
+                      stop(paste('variable', i, ' not found in modelValues') ) 
+                  if(length(j) == 1){
+                      storage.mode(value) <- 'numeric'
+                      eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElement, ptr, as.integer(j), value ))
+                      return(x)
+                  }
+                  for(jj in j)
+                      storage.mode(value[[jj]]) <- 'numeric'
+                  eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElementFromList, ptr, value, as.integer(j)) )
+                  return(x)
+              }
+              cmpNames = names(value)
+              if( !all(cmpNames %in% x$varNames) ) 
+                  stop('Warning: names of modelValue elements do not match')
+              for(n in cmpNames)
+                  x[n, j] <- value[[n]]
+              return(x)
+          })
 
 
 setMethod('[[', 'CmodelValues',
@@ -206,7 +197,7 @@ setMethod('[[<-', 'CmodelValues',
 						eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElement, ptr, as.integer(1), as.numeric(value) ))
 						return(x)
 					}
-				eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElementFromList, ptr, as.numeric(value), as.integer(1:k) ))
+				eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$setMVElementFromList, ptr, value, as.integer(1:k) ))
 				return(x)
 				}
 			cmpNames = names(value)
