@@ -15,7 +15,6 @@
 #' @param scale_param logical; if TRUE the Cholesky factor is that of the scale matrix; otherwise, of the rate matrix.
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details See Gelman et al., Appendix A or the BUGS manual for mathematical details. The rate matrix as used here is defined as the inverse of the scale matrix, \eqn{S^{-1}}, given in Gelman et al. 
 #' @return \code{dwish_chol} gives the density and \code{rwish_chol} generates random deviates.
 #' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
@@ -51,6 +50,55 @@ rwish_chol <- function(n = 1, cholesky, df, scale_param = TRUE) {
     return(out)
 }
 
+#' The Inverse Wishart Distribution
+#'
+#' Density and random generation for the Inverse Wishart distribution, using the Cholesky factor of either the scale matrix or the rate matrix.
+#'
+#' @name Inverse-Wishart
+#' @aliases inverse-wishart
+#' 
+#' @param x vector of values.
+#' @param n number of observations (only \code{n=1} is handled currently).
+#' @param cholesky upper-triangular Cholesky factor of either the scale matrix (when \code{scale_param} is TRUE) or rate matrix (otherwise).
+#' @param df degrees of freedom.
+#' @param scale_param logical; if TRUE the Cholesky factor is that of the scale matrix; otherwise, of the rate matrix.
+#' @param log logical; if TRUE, probability density is returned on the log scale.
+#' @author Christopher Paciorek
+#' @details See Gelman et al., Appendix A for mathematical details. The rate matrix as used here is defined as the inverse of the scale matrix, \eqn{S^{-1}}, given in Gelman et al. 
+#' @return \code{dinvwish_chol} gives the density and \code{rinvwish_chol} generates random deviates.
+#' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
+#' @seealso \link{Distributions} for other standard distributions
+#' 
+#' @examples
+#' df <- 40
+#' ch <- chol(matrix(c(1, .7, .7, 1), 2))
+#' x <- rwish_chol(1, ch, df = df)
+#' dwish_chol(x, ch, df = df)
+#'
+NULL
+
+#' @rdname Inverse-Wishart
+#' @export
+dinvwish_chol <- function(x, cholesky, df, scale_param = TRUE, log = FALSE) {
+  # scale_param = FALSE is the GCSR parameterization (i.e., inverse scale matrix); scale_param = TRUE is the parameterization best for conjugacy calculations (i.e., scale matrix)
+    if(storage.mode(cholesky) != 'double')
+          storage.mode(cholesky) <- 'double'
+    if(storage.mode(x) != 'double')
+            storage.mode(x) <- 'double'
+    .Call('C_dinvwish_chol', x, cholesky, as.double(df), as.double(scale_param), as.logical(log))
+}
+
+#' @rdname Inverse-Wishart
+#' @export
+rinvwish_chol <- function(n = 1, cholesky, df, scale_param = TRUE) {
+    if(n != 1) warning('rinvwish_chol only handles n = 1 at the moment')
+    if(storage.mode(cholesky) != 'double')
+    	storage.mode(cholesky) <- 'double'
+    out <- .Call('C_rinvwish_chol', cholesky, as.double(df), as.double(scale_param))
+    if(!is.null(out)) out <- matrix(out, nrow = sqrt(length(cholesky)))
+    return(out)
+}
+
 
 #' Spectral Decomposition of a Matrix  
 #'
@@ -157,9 +205,57 @@ nimSvd <- function(x, vectors = 'full') {
   .Call('C_nimSvd', x, vectors, svdNimbleList$new())
 }
 
+#' The Improper Uniform Distribution
+#'
+#' Improper flat distribution for use as a prior distribution in BUGS models
+#'
+#' @name flat
+#' @aliases halfflat
+#'
+#' @param x vector of values. 
+#' @param n number of observations.
+#' @param log logical; if TRUE, probability density is returned on the log scale.
+#'
+#' @author Christopher Paciorek
+#' @return \code{dflat} gives the pseudo-density value of 1, while \code{rflat} and \code{rhalfflat} return \code{NaN},
+#' since one cannot simulate from an improper distribution. Similarly, \code{dhalfflat}
+#' gives a pseudo-density value of 1 when \code{x} is non-negative.
+#' @seealso \link{Distributions} for other standard distributions
+#' 
+#' @examples
+#' dflat(1)
+NULL
 
+#' @rdname flat
+#' @export
+dflat <- function(x, log = FALSE) {  
+   if(log) out <- rep(0, length(x)) else  out <- rep(1, length(x))
+   nas <- is.na(x)
+   out[nas] <- x[nas]
+   return(out)
+}
 
+#' @rdname flat
+#' @export
+rflat <- function(n = 1) {
+  return(rep(NaN, n))
+}
 
+#' @rdname flat
+#' @export
+dhalfflat <- function(x, log = FALSE) {
+  out <- rep(0, length(x))
+  out[x < 0] <- -Inf
+  nas <- is.na(x)
+  out[nas] <- x[nas]
+  if(log) return(out) else return(exp(out))
+}
+
+#' @rdname flat
+#' @export
+rhalfflat <- function(n = 1) {
+  return(rep(NaN, n))
+}
 
 #' The Dirichlet Distribution
 #'
@@ -173,7 +269,6 @@ nimSvd <- function(x, vectors = 'full') {
 #' @param alpha vector of parameters of same length as \code{x}
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details See Gelman et al., Appendix A or the BUGS manual for mathematical details. 
 #' @return \code{ddirch} gives the density and \code{rdirch} generates random deviates.
 #' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
@@ -211,7 +306,6 @@ rdirch <- function(n = 1, alpha) {
 #' @param prob vector of probabilities, internally normalized to sum to one, of same length as \code{x}
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details See Gelman et al., Appendix A or the BUGS manual for mathematical details. 
 #' @return \code{dmulti} gives the density and \code{rmulti} generates random deviates.
 #' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
@@ -248,7 +342,6 @@ rmulti <- function(n = 1, size, prob) {
 #' @param prob vector of probabilities, internally normalized to sum to one.
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details See the BUGS manual for mathematical details. 
 #' @return \code{dcat} gives the density and \code{rcat} generates random deviates.
 ##' @seealso \link{Distributions} for other standard distributions
@@ -293,7 +386,6 @@ rcat <- function(n = 1, prob) {
 #' @param log.p logical; if TRUE, probabilities p are given by user as log(p).
 #' @param lower.tail logical; if TRUE (default) probabilities are \eqn{P[X \le x]}; otherwise, \eqn{P[X > x]}.
 #' @author Christopher Paciorek
-#' @export
 #' @details See Gelman et al., Appendix A or the BUGS manual for mathematical details. 
 #' @return \code{dt_nonstandard} gives the density, \code{pt_nonstandard} gives the distribution
 #' function, \code{qt_nonstandard} gives the quantile function, and \code{rt_nonstandard}
@@ -343,7 +435,6 @@ qt_nonstandard <- function(p, df = 1, mu = 0, sigma = 1, lower.tail = TRUE, log.
 #' @param prec_param logical; if TRUE the Cholesky factor is that of the precision matrix; otherwise, of the covariance matrix.
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details See Gelman et al., Appendix A or the BUGS manual for mathematical details. The rate matrix as used here is defined as the inverse of the scale matrix, \eqn{S^{-1}}, given in Gelman et al. 
 #' @return \code{dmnorm_chol} gives the density and \code{rmnorm_chol} generates random deviates.
 #' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
@@ -394,7 +485,6 @@ rmnorm_chol <- function(n = 1, mean, cholesky, prec_param = TRUE) {
 #' @param prec_param logical; if TRUE the Cholesky factor is that of the precision matrix; otherwise, of the scale matrix.
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Peter Sujan
-#' @export
 #' @details See Gelman et al., Appendix A or the BUGS manual for mathematical details. The 'precision' matrix as used here is defined as the inverse of the scale matrix, \eqn{\Sigma^{-1}}, given in Gelman et al. 
 #' @return \code{dmvt_chol} gives the density and \code{rmvt_chol} generates random deviates.
 #' @references Gelman, A., Carlin, J.B., Stern, H.S., and Rubin, D.B. (2004) \emph{Bayesian Data Analysis}, 2nd ed. Chapman and Hall/CRC.
@@ -444,7 +534,6 @@ rmvt_chol <- function(n = 1, mu, cholesky, df, prec_param = TRUE) {
 #' @param c vector of one or more values delineating the intervals.
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details Used for working with censoring in BUGS code.
 #' Taking \code{c} to define the endpoints of two or more intervals (with implicit endpoints of plus/minus infinity), \code{x} (or the return value of \code{rinterval}) gives the non-negative integer valued index of the interval in which \code{t} falls. See the NIMBLE manual for additional details. 
 #' @return \code{dinterval} gives the density and \code{rinterval} generates random deviates,
@@ -484,7 +573,6 @@ rinterval <- function(n = 1, t, c) {
 #' @param cond logical value   
 #' @param log logical; if TRUE, probability density is returned on the log scale.
 #' @author Christopher Paciorek
-#' @export
 #' @details Used for working with constraints in BUGS code.
 #' See the NIMBLE manual for additional details. 
 #' @return \code{dconstraint} gives the density and \code{rconstraint} generates random deviates,
@@ -541,7 +629,6 @@ rconstraint <- function(n = 1, cond) {
 #' @param log.p logical; if TRUE, probabilities p are given by user as log(p).
 #' @param lower.tail logical; if TRUE (default) probabilities are \eqn{P[X \le x]}; otherwise, \eqn{P[X > x]}.
 #' @author Christopher Paciorek
-#' @export
 #' @details NIMBLE's exponential distribution functions use Rmath's functions
 #' under the hood, but are parameterized to take both rate and scale and to
 #' use 'rate' as the core parameterization in C, unlike Rmath, which uses 'scale'.
@@ -622,7 +709,6 @@ qexp_nimble <- function(p, rate = 1/scale, scale = 1, lower.tail = TRUE, log.p =
 #' @param log.p logical; if TRUE, probabilities p are given by user as log(p).
 #' @param lower.tail logical; if TRUE (default) probabilities are \eqn{P[X \le x]}; otherwise, \eqn{P[X > x]}.
 #' @author Christopher Paciorek
-#' @export
 #' @details The inverse gamma distribution with parameters \code{shape} \eqn{=\alpha}{= a} and
 #' \code{scale} \eqn{=\sigma}{= s} has density
 #' \deqn{
@@ -674,6 +760,7 @@ rinvgamma <- function(n = 1, shape, scale = 1, rate = 1/scale) {
     .Call('C_rinvgamma', as.integer(n), as.double(shape), as.double(rate))
 }
 
+
 #' @rdname Inverse-Gamma
 #' @export
 pinvgamma <- function(q, shape, scale = 1, rate = 1/scale, lower.tail = TRUE, log.p = FALSE) {
@@ -696,3 +783,23 @@ qinvgamma <- function(p, shape, scale = 1, rate = 1/scale, lower.tail = TRUE, lo
   .Call('C_qinvgamma', as.double(p), as.double(shape), as.double(rate), as.logical(lower.tail), as.logical(log.p))
 }
 
+# sqrtinvgamma is intended solely for use in conjugacy with dhalfflat
+#' @export
+dsqrtinvgamma <- function(x, shape, scale = 1, rate = 1/scale, log = FALSE) {
+    if (!missing(rate) && !missing(scale)) {
+        if (abs(rate * scale - 1) < 1e-15) 
+            warning("specify 'rate' or 'scale' but not both")
+        else stop("specify 'rate' or 'scale' but not both")
+    }
+    .Call('C_dsqrtinvgamma', as.double(x), as.double(shape), as.double(rate), as.logical(log))
+}
+
+#' @export
+rsqrtinvgamma <- function(n = 1, shape, scale = 1, rate = 1/scale) {
+    if (!missing(rate) && !missing(scale)) {
+        if (abs(rate * scale - 1) < 1e-15) 
+            warning("specify 'rate' or 'scale' but not both")
+        else stop("specify 'rate' or 'scale' but not both")
+    }
+    .Call('C_rsqrtinvgamma', as.integer(n), as.double(shape), as.double(rate))
+}
