@@ -152,6 +152,10 @@ asCol <- function(x) {
 #' @details This is used internally by \code{\link{getParam}}.  It is not intended for direct use by a user or even a nimbleFunction programmer.
 makeParamInfo <- function(model, nodes, param) {
     ## updating to allow nodes to be a vector. getParam only works for a scalar but in a case like nodes[i] the param info is set up for the entire vector.
+
+    ## this allows for(i in seq_along(nodes)) a <- a + model$getParam(nodes[i], 'mean') through compilation even if some instances have nodes empty and so won't be called.
+    if(length(nodes) == 0) return(structure(c(list(paramID = integer()), type = NA, nDim = NA), class = 'getParam_info'))
+
     distNames <- model$getDistribution(nodes)
 
     ## if(length(nodes) != 1) stop(paste0("Problem with nodes argument while setting up getParam.  Should be length 1 but was: ", paste0(nodes, collapse = ",")))
@@ -171,6 +175,12 @@ makeParamInfo <- function(model, nodes, param) {
     ans
 }
 
+defaultParamInfo <- function() {
+    ans <- list(paramID = integer(), type = 'double', nDim = 0)
+    class(ans) <- 'getParam_info'
+    ans
+}
+
 
 #' Get value of a parameter of a stochastic node in a model
 #'
@@ -210,6 +220,7 @@ getParam <- function(model, node, param, nodeFunctionIndex) {
         declID <- indexingInfo$declIDs[1] ## should only be one
         nodeFunction <- model$nodeFunctions[[ declID ]] 
         paramInfo <- makeParamInfo(model, node, param)
+        if(is.na(paramInfo$type)) stop(paste('getParam called with empty or invalid node:', as.character(node)))
     }
     paramID <- paramInfo$paramID
     nDim <- paramInfo$nDim
@@ -361,7 +372,6 @@ rCalcDiffNodes <- function(model, nfv){
 #' @param nodeFunctionIndex For internal NIMBLE use only
 #' @param includeData  A logical argument specifying whether \code{data} nodes should be simulated into (only relevant for \code{\link{simulate}})
 #' @author NIMBLE development team
-#' @export
 #' @details
 #' These functions expands the nodes and then process them in the model in the order provided.  Expanding nodes means turning 'y[1:2]' into c('y[1]','y[2]') if y is a vector of scalar nodes.
 #' Calculation is defined for a stochastic node as executing the log probability (density) calculation and for a deterministic node as calculating whatever function was provided on the right-hand side of the model declaration.
@@ -1094,3 +1104,59 @@ is.nan.vec <- function(x) any(is.nan(x))
 
 #' @export
 nimRound <- round
+
+#' EXPERIMENTAL Nimble wrapper around R's builtin \code{\link{optim}}.
+#'
+#' @param par Initial values for the parameters to be optimized over.
+#' @param fn  A function to be minimized (or maximized), with first argument the
+#'            vector of parameters over which minimization is to take place. It
+#'            should return a scalar result.
+#' @param gr  A function to return the gradient for the "BFGS", "CG" and "L-BFGS-B" methods.
+#'            For the "SANN" method it specifies a function to generate a new candidate point.
+#'            If it is the string "NULL" a default Gaussian Markov kernel is used.
+#' @param ... IGNORED
+#' @param method The method to be used. See \code{Details} section of \code{\link{optim}}. One of:
+#'               "Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN", "Brent"
+#' @param lower Vector or scalar of lower bounds for parameters.
+#' @param upper Vector or scalar of upper bounds for parameters.
+#' @param control A list of control parameters. See \code{Details} section of \code{\link{optim}}.
+#' @param hessian Logical. Should a Hessian matrix be returned?
+#'
+#' @return \code{\link{optimResultNimbleList}}
+#' @seealso \code{\link{optim}}
+#' @export
+nimOptim <- function(par, fn, gr = "NULL", ..., method = "Nelder-Mead", lower = -Inf, upper = Inf,
+                     control = nimOptimDefaultControl(), hessian = FALSE) {
+    # Tweak control value.
+    defaultControl <- optimControlNimbleList$new()
+    Rcontrol <- list()
+    for (name in defaultControl$nimbleListDef$types$vars) {
+        if (!identical(control[[name]], defaultControl[[name]])) {
+            Rcontrol[[name]] <- control[[name]]
+        }
+    }
+    result <- optim(par, fn, gr = gr, ..., method = method,
+                    lower = lower, upper = upper, control = Rcontrol, hessian = hessian)
+    nimResult <- do.call(optimResultNimbleList$new, result)
+    # Tweak result value to exactly match C++ behavior.
+    nimResult$counts <- unname(nimResult$counts)
+    if (is.null(nimResult$message)) {
+        nimResult$message <- ''
+    }
+    if (is.null(nimResult$hessian)) {
+        nimResult$hessian <- matrix(nrow = 0, ncol = 0)
+    }
+    return(nimResult)
+}
+
+#' Creates a deafult \code{control} argument for \code{\link{nimOptim}}.
+#'
+#' @return \code{\link{optimControlNimbleList}}
+#' @seealso \code{\link{nimOptim}}, \code{\link{optim}}
+#' @export
+nimOptimDefaultControl <- function() {
+    control <- optimControlNimbleList$new()
+    control$maxIt <- NULL
+    control$REPORT <- NULL
+    return(control)
+}
