@@ -76,35 +76,6 @@ nf_getSetupOutputNames <- function(f, hidden = FALSE) {
     stop('invalid nimbleFunction argument\n')
 }
 
-nf_getArgOutputNames <- function(f, hidden = FALSE) {
-  nfEnv <- environment(f)
-  methodList <- nfEnv$methodList
-  methodArgListCode <- lapply(methodList, function(x){
-    outputType <-  as.character(x$argInfo[[1]][[1]])
-    if((length(outputType) == 0) ||
-       outputType %in% c('double', 'integer', 'character', 'logical', 'internalType'))
-      return(NULL)
-    else
-      return(outputType)
-  })
-  return(unlist(methodArgListCode))
-}
-
-nf_getReturnTypeOutputNames <- function(f, hidden = FALSE) {
-  nfEnv <- environment(f)
-  methodList <- nfEnv$methodList
-  methodReturnTypes <- lapply(methodList, function(x){ RT <- as.character(x$returnType)[1]
-                                                       ## vector of types below should contain all basic types, as well as nlDefs that we
-                                                       ## do keyword replacement on (eigen, svd, ...)
-                                                       # if(!RT %in%  c('double', 'integer', 'character', 'logical', 'internalType', 'void',
-                                                       #                'eigen', 'svd'))
-                                                        if(!RT %in%  c('double', 'integer', 'character', 'logical', 'void',
-                                                                       'nimEigen', 'nimSvd'))
-                                                         return(RT)
-                                                       else return(NULL)})
-  return(unlist(methodReturnTypes))
-}
-
 #'
 #' Get nimbleFunction definition
 #'
