@@ -24,6 +24,16 @@ void multivarTestCall(double *x, int n) {
   nimble_print_to_R(_nimble_global_output);
 }
 
+vector<int> getSEXPdims(SEXP Sx) {
+  if(!isNumeric(Sx)) {PRINTF("Error, getSEXPdims called for something not numeric\n"); return(vector<int>());}
+  if(!isVector(Sx)) {PRINTF("Error, getSEXPdims called for something not vector\n"); return(vector<int>());}
+  if(!isArray(Sx) & !isMatrix(Sx)) {
+    vector<int> ans; 
+    ans.resize(1); ans[0] = LENGTH(Sx); return(ans);
+  }
+  return(SEXP_2_vectorInt(getAttrib(Sx, R_DimSymbol), 0));
+}
+
 string STRSEXP_2_string(SEXP Ss, int i) {
   if(!isString(Ss)) {
     PRINTF("Error: STRSEXP_2_string called for SEXP that is not a string!\n"); 
@@ -135,6 +145,26 @@ SEXP vectorInt_2_SEXP(const vector<int> &v) {
   return(Sans);
 }
 
+
+void vectorDouble_2_SEXP(const vector<double> &v, SEXP Sn) {
+  int nn = v.size();
+  PROTECT(Sn = allocVector(REALSXP, nn));
+  if(nn > 0) {
+    copy(v.begin(), v.end(), REAL(Sn));
+  }
+  UNPROTECT(1);
+}
+
+void vectorInt_2_SEXP(const vector<int> &v, SEXP Sn) {
+  int nn = v.size();
+  PROTECT(Sn = allocVector(INTSXP, nn));
+  if(nn > 0) {
+    copy(v.begin(), v.end(), INTEGER(Sn));
+  }
+  UNPROTECT(1);
+}
+
+
 struct opIntegerShift {
 public:
   int c;
@@ -156,6 +186,8 @@ SEXP vectorInt_2_SEXP(const vector<int> &v, int offset) {
   return(Sans);
 }
 
+
+
 vector<int> SEXP_2_vectorInt( SEXP Sn, int offset ) {
   if(!(isNumeric(Sn) || isLogical(Sn))) PRINTF("Error: SEXP_2_vectorInt called for SEXP that is not a numeric or logica!\n");
   int nn = LENGTH(Sn);
@@ -334,7 +366,7 @@ bool SEXP_2_bool(SEXP Sn, int i) {
 // 	return;
 // }
 
-SEXP SEXP_2_double(SEXP rPtr, SEXP refNum, SEXP rScalar){
+SEXP populate_SEXP_2_double(SEXP rPtr, SEXP refNum, SEXP rScalar){
     void* vPtr = R_ExternalPtrAddr(rPtr);
     if(vPtr == NULL){
         PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -359,7 +391,7 @@ SEXP SEXP_2_double(SEXP rPtr, SEXP refNum, SEXP rScalar){
     return(R_NilValue);
 }
 
-SEXP SEXP_2_int(SEXP rPtr, SEXP refNum, SEXP rScalar){
+SEXP populate_SEXP_2_int(SEXP rPtr, SEXP refNum, SEXP rScalar){
     void* vPtr = R_ExternalPtrAddr(rPtr);
     if(vPtr == NULL){
         PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -383,7 +415,7 @@ SEXP SEXP_2_int(SEXP rPtr, SEXP refNum, SEXP rScalar){
 		//But our generated code calls both
 
 
-  SEXP SEXP_2_bool(SEXP rPtr, SEXP refNum, SEXP rScalar){
+  SEXP populate_SEXP_2_bool(SEXP rPtr, SEXP refNum, SEXP rScalar){
     void* vPtr = R_ExternalPtrAddr(rPtr);
     if(vPtr == NULL){
         PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -407,7 +439,7 @@ SEXP SEXP_2_int(SEXP rPtr, SEXP refNum, SEXP rScalar){
     return(R_NilValue);
 }
 
-SEXP bool_2_SEXP(SEXP rPtr, SEXP refNum){
+SEXP extract_bool_2_SEXP(SEXP rPtr, SEXP refNum){
     void* vPtr = R_ExternalPtrAddr(rPtr);
     if(vPtr == NULL){
         PRINTF("Warning: pointing to NULL in bool_2_SEXP\n");
@@ -430,10 +462,10 @@ SEXP bool_2_SEXP(SEXP rPtr, SEXP refNum){
     return(Sans);
 }
 
-SEXP double_2_SEXP(SEXP rPtr, SEXP refNum){
+SEXP extract_double_2_SEXP(SEXP rPtr, SEXP refNum){
     void* vPtr = R_ExternalPtrAddr(rPtr);
     if(vPtr == NULL){
-        PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
+        PRINTF("Warning: pointing to NULL in extract_double_2_SEXP\n");
         return(R_NilValue);
     }
     double* cPtr;
@@ -443,7 +475,7 @@ SEXP double_2_SEXP(SEXP rPtr, SEXP refNum){
     else if(cRefNum == 2)
         cPtr = (*static_cast<double**> ( vPtr ) );
     else {
-      PRINTF("Warning: double_2_SEXP called with reNum != 1 or 2\n");
+      PRINTF("Warning: extract_double_2_SEXP called with reNum != 1 or 2\n");
       return(R_NilValue);
     }
     SEXP Sans;
@@ -453,8 +485,7 @@ SEXP double_2_SEXP(SEXP rPtr, SEXP refNum){
     return(Sans);
 }
 
-
-SEXP int_2_SEXP(SEXP rPtr, SEXP refNum){
+SEXP extract_int_2_SEXP(SEXP rPtr, SEXP refNum){
     void* vPtr = R_ExternalPtrAddr(rPtr);
     if(vPtr == NULL){
         PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -477,7 +508,7 @@ SEXP int_2_SEXP(SEXP rPtr, SEXP refNum){
     return(Sans);
 }
 
-SEXP SEXP_2_string(SEXP rPtr, SEXP rString) {
+SEXP populate_SEXP_2_string(SEXP rPtr, SEXP rString) {
   void* vPtr = R_ExternalPtrAddr(rPtr);
   if(vPtr == NULL){
     PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -487,7 +518,7 @@ SEXP SEXP_2_string(SEXP rPtr, SEXP rString) {
   return(R_NilValue);
 }
 
-SEXP SEXP_2_stringVector(SEXP rPtr, SEXP rStringVector) {
+SEXP populate_SEXP_2_stringVector(SEXP rPtr, SEXP rStringVector) {
   void* vPtr = R_ExternalPtrAddr(rPtr);
   if(vPtr == NULL){
     PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -497,7 +528,7 @@ SEXP SEXP_2_stringVector(SEXP rPtr, SEXP rStringVector) {
   return(R_NilValue);
 }
 
-SEXP string_2_SEXP(SEXP rPtr) {
+SEXP extract_string_2_SEXP(SEXP rPtr) {
   void* vPtr = R_ExternalPtrAddr(rPtr);
   if(vPtr == NULL){
     PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -506,7 +537,7 @@ SEXP string_2_SEXP(SEXP rPtr) {
   return(string_2_STRSEXP(*static_cast<string *>(vPtr)));
 }
 
-SEXP stringVector_2_SEXP(SEXP rPtr) {
+SEXP extract_stringVector_2_SEXP(SEXP rPtr) {
   void* vPtr = R_ExternalPtrAddr(rPtr);
   if(vPtr == NULL){
     PRINTF("Warning: pointing to NULL in SEXP_2_double\n");
@@ -645,8 +676,8 @@ void rawSample(double* p, int c_samps, int N, int* ans, bool unsort, bool silent
   }
 }
 
-SEXP rankSample(SEXP p, SEXP n, SEXP not_used, SEXP s) {
-  //PRINTF("in SEXP rankSample\n");
+SEXP C_rankSample(SEXP p, SEXP n, SEXP not_used, SEXP s) {
+  //PRINTF("in SEXP C_rankSample\n");
   int N = LENGTH(p);
   int c_samps = INTEGER(n)[0];
   bool silent = LOGICAL(s)[0];
@@ -683,3 +714,15 @@ SEXP parseVar(SEXP Sinput) {
   parseVar(input, output);
   return(vectorString_2_STRSEXP(output));
 }
+
+
+SEXP makeNewNimbleList(SEXP S_listName) {
+  SEXP SnimbleInternalFunctionsEnv;
+  SEXP call;
+  PROTECT(SnimbleInternalFunctionsEnv = EVAL(findVar(install("nimbleInternalFunctions"), R_GlobalEnv)));
+  PROTECT(call = allocVector(LANGSXP, 2));
+  SETCAR(call, install("makeNewNimListSEXPRESSIONFromC"));
+  SETCADR(call, S_listName);
+  UNPROTECT(2);
+  return(eval(call, SnimbleInternalFunctionsEnv));
+}
