@@ -14,9 +14,13 @@ sapply(models, testBUGSmodel, useInits = TRUE)
 # otherwise p[i,j] is a standard conjugate update
 
 # lsat has non-explicit indexing
-system(paste("cat", system.file('classic-bugs','vol1','lsat','lsat.bug', package = 'nimble'), ">>", file.path(tempdir(), "lsat.bug")))
-system(paste("sed -i -e 's/mean(alpha\\[\\])/mean(alpha\\[1:T\\])/g'", file.path(tempdir(), "lsat.bug"))) 
-system(paste("sed -i -e 's/p.item\\[i,\\]/p.item\\[i,1:T\\]/g'", file.path(tempdir(), "lsat.bug"))) 
+## changes for windows compatibility
+file.copy( system.file('classic-bugs','vol1','lsat','lsat.bug', package = 'nimble'), file.path(tempdir(), "lsat.bug"))
+##system(paste("cat", system.file('classic-bugs','vol1','lsat','lsat.bug', package = 'nimble'), ">>", file.path(tempdir(), "lsat.bug")))
+system.in.dir("sed -i -e 's/mean(alpha\\[\\])/mean(alpha\\[1:T\\])/g' lsat.bug", dir = tempdir())
+##system(paste("sed -i -e 's/mean(alpha\\[\\])/mean(alpha\\[1:T\\])/g'", file.path(tempdir(), "lsat.bug")))
+system.in.dir("sed -i -e 's/p.item\\[i,\\]/p.item\\[i,1:T\\]/g' lsat.bug", dir = tempdir())
+##system(paste("sed -i -e 's/p.item\\[i,\\]/p.item\\[i,1:T\\]/g'", file.path(tempdir(), "lsat.bug")))
 testBUGSmodel('lsat', dir = "", model = file.path(tempdir(), "lsat.bug"), data = system.file('classic-bugs','vol1','lsat','lsat-data.R', package = 'nimble'),  inits = system.file('classic-bugs','vol1','lsat','lsat-init.R', package = 'nimble'),  useInits = TRUE)
 
 
@@ -51,7 +55,7 @@ writeLines(c("model {",
            con = filename)
 inits <- list(mu = 2)
 data <- list(x = -2)
-testBUGSmodel(model = filename, dir = '', 
+testBUGSmodel(model = filename, dir = '',
               data = data, inits = inits)
 
 # non-truncated T() test
@@ -102,14 +106,14 @@ inits <- list(mu = 1.5)
 
 Rmodel = nimbleModel(code, data = data, inits = inits,
     constants = constants)
-mcmcspec <- configureMCMC(Rmodel)
-Rmcmc <- buildMCMC(mcmcspec)
+mcmcConf <- configureMCMC(Rmodel)
+Rmcmc <- buildMCMC(mcmcConf)
 Cmodel <- compileNimble(Rmodel)
 Cmcmc <- compileNimble(Rmcmc, project = Rmodel)
 Cmcmc$run(5000)
 smp <- as.matrix(Cmcmc$mvSamples)
-try(test_that("Test that MCMC respects truncation bounds: ", expect_that(min(smp), is_more_than(0.2), info = "minimum in MCMC greater than lower bound")))
-try(test_that("Test that MCMC respects truncation bounds: ", expect_that(min(smp), is_less_than(2), info = "maximum in MCMC less than upper bound")))
+try(test_that("Test that MCMC respects truncation bounds: minimum in MCMC greater than lower bound", expect_gt(min(smp), 0.2)))
+try(test_that("Test that MCMC respects truncation bounds: maximum in MCMC less than upper bound", expect_lt(min(smp), 2)))
 
 test_mcmc(model = code, data = c(data, constants), inits = inits,
           results = list(mean = list(mu = 1.5), sd = list(mu = .27 )),
@@ -143,16 +147,16 @@ inits <- list(mu = mu, bnd = bnd, y = yinits)
 
 Rmodel = nimbleModel(code, data = data, inits = inits,
     constants = constants)
-mcmcspec <- configureMCMC(Rmodel)
-mcmcspec$addMonitors('y[1]')
-Rmcmc <- buildMCMC(mcmcspec)
+mcmcConf <- configureMCMC(Rmodel)
+mcmcConf$addMonitors('y[1]')
+Rmcmc <- buildMCMC(mcmcConf)
 Cmodel <- compileNimble(Rmodel)
 Cmcmc <- compileNimble(Rmcmc, project = Rmodel)
 Cmcmc$run(5000)
 smp <- as.matrix(Cmcmc$mvSamples)
-try(test_that("Test that MCMC respects right-censoring bounds: ", expect_that(min(smp[ , 'bnd'] - max(y[!is.cens])), is_more_than(0), info = "upper bound in MCMC exceeds observations")))
-try(test_that("Test that MCMC respects censoring bounds: ",
-              expect_that(min(smp[ , 'y[1]'] - smp[ , 'bnd']), is_more_than(0), info = "minimum inferred censored value in MCMC greater than bound")))
+try(test_that("Test that MCMC respects right-censoring bounds: upper bound in MCMC exceeds observations", expect_gt(min(smp[ , 'bnd'] - max(y[!is.cens])), 0)))
+try(test_that("Test that MCMC respects censoring bounds: minimum inferred censored value in MCMC greater than bound",
+              expect_gt(min(smp[ , 'y[1]'] - smp[ , 'bnd']), 0)))
 
 
 test_mcmc(model = code, data = c(data, constants), inits = inits, numItsC = 10000,
@@ -186,16 +190,16 @@ inits <- list(mu = mu, bnd = bnd, y = yinits)
 
 Rmodel = nimbleModel(code, data = data, inits = inits,
     constants = constants)
-mcmcspec <- configureMCMC(Rmodel)
-mcmcspec$addMonitors('y[1]')
-Rmcmc <- buildMCMC(mcmcspec)
+mcmcConf <- configureMCMC(Rmodel)
+mcmcConf$addMonitors('y[1]')
+Rmcmc <- buildMCMC(mcmcConf)
 Cmodel <- compileNimble(Rmodel)
 Cmcmc <- compileNimble(Rmcmc, project = Rmodel)
 Cmcmc$run(5000)
 smp <- as.matrix(Cmcmc$mvSamples)
-try(test_that("Test that MCMC respects left-censoring bounds: ", expect_that(min(min(y[not.cens]) - smp[ , 'bnd']), is_more_than(0), info = "lower bound in MCMC is less than observations")))
-try(test_that("Test that MCMC respects censoring bounds: ",
-              expect_that(max(smp[ , 'bnd'] - smp[ , 'y[1]']), is_more_than(0), info = "maximum inferred censored value in MCMC less than bound")))
+try(test_that("Test that MCMC respects left-censoring bounds: lower bound in MCMC is less than observations", expect_gt(min(min(y[not.cens]) - smp[ , 'bnd']), 0)))
+try(test_that("Test that MCMC respects censoring bounds: maximum inferred censored value in MCMC less than bound",
+              expect_gt(max(smp[ , 'bnd'] - smp[ , 'y[1]']), 0)))
 
 test_mcmc(model = code, data = c(data, constants), inits = inits,
           results = list(mean = list(mu = 43.4, 'y[1]' = 33), sd = list(mu = 2.4, 'y[1]' = 5.3)),
@@ -226,9 +230,9 @@ yinits[is.na(y)] <- c(rep(35, 4), rep(45, 4), rep(55, 4))
 inits <- list(mu = mu, y = yinits)
 
 Rmodel = nimbleModel(code, data = data, inits = inits, constants = constants)
-mcmcspec <- configureMCMC(Rmodel)
-mcmcspec$addMonitors(c('y[2]', 'y[7]', 'y[12]'))
-Rmcmc <- buildMCMC(mcmcspec)
+mcmcConf <- configureMCMC(Rmodel)
+mcmcConf$addMonitors(c('y[2]', 'y[7]', 'y[12]'))
+Rmcmc <- buildMCMC(mcmcConf)
 Cmodel <- compileNimble(Rmodel)
 Cmcmc <- compileNimble(Rmcmc, project = Rmodel)
 Cmcmc$run(5000)
@@ -252,7 +256,7 @@ code <- nimbleCode ({
     }
     mu1 ~ dunif(-10, 10)
     mu2 ~ dunif(-10, 10)
-    mu.c ~ dconstraint(mu1 + mu2 > 0 & mu1 > 1) 
+    mu.c ~ dconstraint(mu1 + mu2 > 0 & mu1 > 1)
 })
 mu1 <- 1; mu2 <- -0.5
 n <- 10
@@ -263,14 +267,14 @@ inits = list(mu1 = 1.5, mu2 = 0.5)
 constants = list(n = n)
 
 Rmodel = nimbleModel(code, data = data, inits = inits, constants = constants)
-mcmcspec <- configureMCMC(Rmodel)
-Rmcmc <- buildMCMC(mcmcspec)
+mcmcConf <- configureMCMC(Rmodel)
+Rmcmc <- buildMCMC(mcmcConf)
 Cmodel <- compileNimble(Rmodel)
 Cmcmc <- compileNimble(Rmcmc, project = Rmodel)
 Cmcmc$run(5000)
 smp <- as.matrix(Cmcmc$mvSamples)
-try(test_that("Test that MCMC respects constraints: ", expect_that(min(smp[ , 'mu1'] + smp[ , 'mu2']), is_more_than(0), info = "constraint on sum of mu1 and mu2 is violated")))
-try(test_that("Test that MCMC respects constraints: ", expect_that(min(smp[ , 'mu1']), is_more_than(0), info = "constraint on mu1 is violated")))
+try(test_that("Test that MCMC respects constraints: constraint on sum of mu1 and mu2 is violated", expect_gt(min(smp[ , 'mu1'] + smp[ , 'mu2']), 0)))
+try(test_that("Test that MCMC respects constraints: constraint on mu1 is violated", expect_gt(min(smp[ , 'mu1']), 0)))
 
 test_mcmc(model = code, data = c(data, constants), inits = inits,
           results = list(mean = list(mu1 = 1.45, mu2 = -.82), sd = list(mu1 = .26, mu2 = .30)),
@@ -287,10 +291,10 @@ code <- nimbleCode({
     for (i in 1:N) {
         for (t in 1:T) {
             for (j in 1:Ncut) {
-#  
+#
 # Cumulative probability of worse response than j
 #
-                logit(Q[i,t,j]) <- -(a0[j] + mu[group[i],t] + b[i]); 
+                logit(Q[i,t,j]) <- -(a0[j] + mu[group[i],t] + b[i]);
             }
 #
 # Probability of response = j
@@ -298,7 +302,7 @@ code <- nimbleCode({
             p[i,t,1] <- 1 - Q[i,t,1];
             for (j in 2:Ncut) { p[i,t,j] <- Q[i,t,j-1] - Q[i,t,j] }
             p[i,t,(Ncut+1)] <- Q[i,t,Ncut];
-            
+
             response[i,t] ~ dcat(p[i,t,1:4]);
         }
 #
@@ -306,26 +310,26 @@ code <- nimbleCode({
 #
         b[i] ~ dnorm(0.0, tau);
     }
-    
+
 #
 # Fixed effects
 #
     for (g in 1:G) {
-        for(t in 1:T) { 
+        for(t in 1:T) {
                                         # logistic mean for group i in period t
-            mu[g,t] <- beta*treat[g,t]/2 + pi*period[g,t]/2 + kappa*carry[g,t]; 
+            mu[g,t] <- beta*treat[g,t]/2 + pi*period[g,t]/2 + kappa*carry[g,t];
         }
-    }                                                             
+    }
     beta ~ dnorm(0, 1.0E-06);
     pi ~ dnorm(0, 1.0E-06);
     kappa ~ dnorm(0, 1.0E-06);
-    
-# ordered cut points for underlying continuous latent variable  
+
+# ordered cut points for underlying continuous latent variable
     ordered ~ dconstraint(a0[1] < a0[2] & a0[2] < a0[3]);
     for(i in 1:3) {
         a0[i] ~ dnorm(0, 1.0E-6);
     }
-    
+
     tau ~ dgamma(0.001, 0.001);
     sigma <- sqrt(1/tau);
     log.sigma <- log(sigma);
@@ -354,21 +358,21 @@ response <- matrix(c(rep(pattern[1, ], 59+63),
                      rep(pattern[11, ], 0+1),
                      rep(pattern[13, ], 1+2),
                      rep(pattern[14, ], 1+0),
-                     rep(pattern[15, ], 0+1)),                  
+                     rep(pattern[15, ], 0+1)),
                    ncol = 2, byrow = TRUE)
 
 constants$group = group
 data = list(ordered = 1, response = response)
 
 Rmodel = nimbleModel(code, data = data, inits = as.list(inits), constants = as.list(constants))
-mcmcspec <- configureMCMC(Rmodel)
-Rmcmc <- buildMCMC(mcmcspec)
+mcmcConf <- configureMCMC(Rmodel)
+Rmcmc <- buildMCMC(mcmcConf)
 Cmodel <- compileNimble(Rmodel)
 Cmcmc <- compileNimble(Rmcmc, project = Rmodel)
 Cmcmc$run(5000)
 smp <- as.matrix(Cmcmc$mvSamples)
-try(test_that("Test that MCMC respects constraints: ", expect_that(max(smp[ , 'a0[1]'] - smp[ , 'a0[2]']), is_less_than(0), info = "constraint on ordering of a[1] and a[2] is violated")))
-try(test_that("Test that MCMC respects constraints: ", expect_that(max(smp[ , 'a0[2]'] - smp[ , 'a0[3]']), is_less_than(0), info = "constraint on ordering of a[2] and a[3] is violated")))
+try(test_that("Test that MCMC respects constraints: constraint on ordering of a[1] and a[2] is violated", expect_lt(max(smp[ , 'a0[1]'] - smp[ , 'a0[2]']), 0)))
+try(test_that("Test that MCMC respects constraints: constraint on ordering of a[2] and a[3] is violated", expect_lt(max(smp[ , 'a0[2]'] - smp[ , 'a0[3]']), 0)))
 
 test_mcmc(model = code, inits = as.list(inits), data = c(data, as.list(constants)), numItsC = 10000,
           resampleData = TRUE, basic = FALSE,
@@ -394,11 +398,11 @@ code <- nimbleCode({
 })
 
 m <- nimbleModel(code, data = list(y = 1), inits = list(mu2 = 1))
-spec <- configureMCMC(m)
+conf <- configureMCMC(m)
 
-try(test_that("Test that MCMC with truncation avoids conjugate samplers: ", expect_that(spec$getSamplers('mu1')[[1]]$name, is_identical_to('RW'), info = "incorrectly assigning conjugate sampler for mu1")))
-try(test_that("Test that MCMC with truncation avoids conjugate samplers: ", expect_that(spec$getSamplers('mu2')[[1]]$name, is_identical_to('RW'), info = "incorrectly assigning conjugate sampler for mu2")))
-try(test_that("Test that MCMC with truncation avoids conjugate samplers: ", expect_that(spec$getSamplers('mu3')[[1]]$name, is_identical_to('conjugate_dnorm_dnorm'), info = "incorrectly not assigning conjugate sampler for mu3")))
+try(test_that("Test that MCMC with truncation avoids conjugate samplers: ", expect_identical(conf$getSamplers('mu1')[[1]]$name, 'RW', info = "incorrectly assigning conjugate sampler for mu1")))
+try(test_that("Test that MCMC with truncation avoids conjugate samplers: ", expect_identical(conf$getSamplers('mu2')[[1]]$name, 'RW', info = "incorrectly assigning conjugate sampler for mu2")))
+try(test_that("Test that MCMC with truncation avoids conjugate samplers: ", expect_identical(conf$getSamplers('mu3')[[1]]$name, 'conjugate_dnorm_dnorm', info = "incorrectly not assigning conjugate sampler for mu3")))
    
 # test that truncation on discrete distribution correctly uses [L, U]
 # and test use with alias too
@@ -446,48 +450,48 @@ for(j in 1:5) {
 
 diffProbsR <- abs(simProbs - c(dens_y123[1], dens_y123[1], dens_y123[1], dens_y4[1], dens_y5[1]))
 
-try(test_that("Test that truncation with discrete distribution gives correct simulation: ", expect_that(max(diffProbsR), is_less_than(0.015), info = "simulation does not give approximate density values")))
+try(test_that("Test that truncation with discrete distribution gives correct simulation: simulation does not give approximate density values", expect_lt(max(diffProbsR), 0.015)))
 
 m$y[1] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[1]'), equals(log(dens_y123[1])), info = "incorrect R model density for y[1]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[1]'), (log(dens_y123[1])), info = "incorrect R model density for y[1]=1")))
 m$y[1] <- 2
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[1]'), equals(log(dens_y123[2])), info = "incorrect R model density for y[1]=2")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[1]'), (log(dens_y123[2])), info = "incorrect R model density for y[1]=2")))
 m$y[2] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[2]'), equals(log(dens_y123[1])), info = "incorrect R model density for y[2]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[2]'), (log(dens_y123[1])), info = "incorrect R model density for y[2]=1")))
 m$y[2] <- 2
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[2]'), equals(log(dens_y123[2])), info = "incorrect R model density for y[2]=2")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[2]'), (log(dens_y123[2])), info = "incorrect R model density for y[2]=2")))
 m$y[3] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[3]'), equals(log(dens_y123[1])), info = "incorrect R model density for y[3]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[3]'), (log(dens_y123[1])), info = "incorrect R model density for y[3]=1")))
 m$y[3] <- 2
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[3]'), equals(log(dens_y123[2])), info = "incorrect R model density for y[3]=2")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[3]'), (log(dens_y123[2])), info = "incorrect R model density for y[3]=2")))
 m$y[4] <- 0
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[4]'), equals(log(dens_y4[1])), info = "incorrect R model density for y[4]=0")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[4]'), (log(dens_y4[1])), info = "incorrect R model density for y[4]=0")))
 m$y[4] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[4]'), equals(log(dens_y4[2])), info = "incorrect R model density for y[4]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[4]'), (log(dens_y4[2])), info = "incorrect R model density for y[4]=1")))
 m$y[5] <- n-1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[5]'), equals(log(dens_y5[1])), info = "incorrect R model density for y[5]=n-1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[5]'), (log(dens_y5[1])), info = "incorrect R model density for y[5]=n-1")))
 m$y[5] <- n
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(m, 'y[5]') , equals(log(dens_y5[2])), info = "incorrect R model density for y[5]=n")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(m, 'y[5]') , (log(dens_y5[2])), info = "incorrect R model density for y[5]=n")))
 
 cm$y[1] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[1]'), equals(log(dens_y123[1])), info = "incorrect C model density for y[1]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[1]'), (log(dens_y123[1])), info = "incorrect C model density for y[1]=1")))
 cm$y[1] <- 2
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[1]'), equals(log(dens_y123[2])), info = "incorrect C model density for y[1]=2")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[1]'), (log(dens_y123[2])), info = "incorrect C model density for y[1]=2")))
 cm$y[2] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[2]'), equals(log(dens_y123[1])), info = "incorrect C model density for y[2]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[2]'), (log(dens_y123[1])), info = "incorrect C model density for y[2]=1")))
 cm$y[2] <- 2
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[2]'), equals(log(dens_y123[2])), info = "incorrect C model density for y[2]=2")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[2]'), (log(dens_y123[2])), info = "incorrect C model density for y[2]=2")))
 cm$y[3] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[3]'), equals(log(dens_y123[1])), info = "incorrect C model density for y[3]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[3]'), (log(dens_y123[1])), info = "incorrect C model density for y[3]=1")))
 cm$y[3] <- 2
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[3]'), equals(log(dens_y123[2])), info = "incorrect C model density for y[3]=2")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[3]'), (log(dens_y123[2])), info = "incorrect C model density for y[3]=2")))
 cm$y[4] <- 0
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[4]'), equals(log(dens_y4[1])), info = "incorrect C model density for y[4]=0")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[4]'), (log(dens_y4[1])), info = "incorrect C model density for y[4]=0")))
 cm$y[4] <- 1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[4]'), equals(log(dens_y4[2])), info = "incorrect C model density for y[4]=1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[4]'), (log(dens_y4[2])), info = "incorrect C model density for y[4]=1")))
 cm$y[5] <- n-1
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[5]'), equals(log(dens_y5[1])), info = "incorrect C model density for y[5]=n-1")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[5]'), (log(dens_y5[1])), info = "incorrect C model density for y[5]=n-1")))
 cm$y[5] <- n
-try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_that(calculate(cm, 'y[5]'), equals(log(dens_y5[2])), info = "incorrect C model density for y[5]=n")))
+try(test_that("Test that truncation with discrete distribution gives correct density: ", expect_equal(calculate(cm, 'y[5]'), (log(dens_y5[2])), info = "incorrect C model density for y[5]=n")))
 
 
