@@ -18,6 +18,14 @@ system.in.dir <- function(cmd, dir = '.') {
         system(cmd)
 }
 
+## This is useful for working around scoping issues with nimbleFunctions using other nimbleFunctions.
+temporarilyAssignInGlobalEnv <- function(value) {
+    name <- deparse(substitute(value))
+    assign(name, value, envir = .GlobalEnv)
+    rmCommand <- substitute(remove(name, envir = .GlobalEnv))
+    do.call('on.exit', list(rmCommand), envir = parent.frame())
+}
+
 gen_runFunCore <- function(input) {
     runFun <- function() {}
     formalsList <- input$args
@@ -28,7 +36,10 @@ gen_runFunCore <- function(input) {
     formals(runFun) <- formalsList
     tmp <- quote({})
     tmp[[2]] <- input$expr
-    tmp[[3]] <- quote(return(out))
+    tmp[[3]] <- if(is.null(input[['return']]))
+                    quote(return(out))
+                else
+                    input[['return']]
     tmp[[4]] <- substitute(returnType(OUT), list(OUT = input$outputType))
     body(runFun) <- tmp
     return(runFun)
@@ -122,21 +133,31 @@ test_coreRfeature <- function(input, verbose = TRUE, dirName = NULL) { ## a lot
   attr(out_nfC, 'dim') <- dimOutC
   checkEqual <- input[['checkEqual']]
   if(is.null(checkEqual)) checkEqual <- FALSE
-  if(!checkEqual) {
-      try(test_that(paste0("Identical test of coreRfeature (direct R vs. R nimbleFunction): ", input$name),
-                    expect_identical(out, out_nfR)))
-      try(test_that(paste0("Identical test of math (direct R vs. C++ nimbleFunction): ", input$name),
-                    expect_identical(out, out_nfC)))
-  } else {
-      try(test_that(paste0("Equal test of coreRfeature (direct R vs. R nimbleFunction): ", input$name),
-                    expect_equal(out, out_nfR)))
-      try(test_that(paste0("Equal test of math (direct R vs. C++ nimbleFunction): ", input$name),
-                    expect_equal(out, out_nfC)))
+  if(is.null(input[['return']])) { ## use default 'out' object
+      if(!checkEqual) {
+          try(test_that(paste0("Identical test of coreRfeature (direct R vs. R nimbleFunction): ", input$name),
+                        expect_identical(out, out_nfR)))
+          try(test_that(paste0("Identical test of math (direct R vs. C++ nimbleFunction): ", input$name),
+                        expect_identical(out, out_nfC)))
+      } else {
+          try(test_that(paste0("Equal test of coreRfeature (direct R vs. R nimbleFunction): ", input$name),
+                        expect_equal(out, out_nfR)))
+          try(test_that(paste0("Equal test of math (direct R vs. C++ nimbleFunction): ", input$name),
+                        expect_equal(out, out_nfC)))
+      }
+  } else { ## not using default return(out), so only compare out_nfR to out_nfC
+      if(!checkEqual) {
+          try(test_that(paste0("Identical test of coreRfeature (compiled vs. uncompied nimbleFunction): ", input$name),
+                        expect_identical(out_nfC, out_nfR)))
+      } else {
+          try(test_that(paste0("Equal test of coreRfeature (compiled vs. uncompied nimbleFunction): ", input$name),
+                        expect_equal(out_nfC, out_nfR)))
+
+      }
   }
   # unload DLL as R doesn't like to have too many loaded
   if(.Platform$OS.type != 'windows') nimble:::clearCompiled(nfR) ##dyn.unload(project$cppProjects[[1]]$getSOName())
   invisible(NULL)
-
 }
 
 gen_runFun <- function(input) {
@@ -838,3 +859,32 @@ test_getBound <- function(model, cmodel, test, node, bnd, truth, info) {
     invisible(NULL)
 }
  
+## utilities for saving test output to a reference file
+## and making the test a comparison of the file
+clearOldOutput <- function(filename) {
+    if(file.exists(filename)) file.remove(filename)
+}
+
+appendOutput <- function(filename, case, caseName, casePrefix = "") {
+    outputConnection <- file(filename, open = 'at')
+    writeLines(caseName, con = outputConnection)
+    outputAns <- lapply(case, function(x) writeLines(paste0(casePrefix, paste(x, collapse = " ")), con = outputConnection))
+    close(outputConnection)
+}
+
+writeOutput <- function(cases, filename) {
+    clearOldOutput(filename)
+    for(i in seq_along(cases)) appendOutput(filename, cases[[i]], names(cases)[i], casePrefix = paste0(i,": "))
+}
+
+compareFilesByLine <- function(trialResults, correctResults, main = "") {
+    test_that(paste0(main, ': same number of output lines'),
+          expect_equal(length(trialResults), length(correctResults)))
+    
+    linesToTest <- min(length(trialResults), length(correctResults))
+    mapply(function(lineno, trialLine, correctLine) {
+        test_that(paste0(main, ": output line #", lineno),
+                  expect_identical(trialLine, correctLine))
+    }, 1:linesToTest, trialResults, correctResults)
+    invisible(NULL)
+}
